@@ -270,22 +270,17 @@ class TablasVC(SQLConnector):
         
         return dict(zip(llaves,valores))
  
-    def __crear_dict_tipo_empresa(self):
-        flt_bop = ~self.empresas['mantenimientoBOP'].isna()
-        flt_gen = ~self.empresas['generador'].isna() 
-        flt_opL = ~self.empresas['operacionLocal'].isna()
-        flt_mantL = ~self.empresas['mantenimientoparque'].isna() 
-        flt_traspo = ~self.empresas['transportista'].isna()
-        flt_distro = ~self.empresas['distribuidora'].isna()
-        flt_adm = ~self.empresas['administracion'].isna()
+    def _crear_dict_tipo_empresa(self,devolver=False):
+        self._empresas_bop = self.empresas.Empresa[self.empresas.mantenimientoBOP.notna()].str.upper()
+        self._empresas_gen = self.empresas.Empresa[self.empresas.generador.notna() | self.empresas.operacionLocal.notna() | self.empresas.mantenimientoparque.notna()].str.upper()
+        self._empresas_grid = self.empresas.Empresa[self.empresas.transportista.notna() | self.empresas.distribuidora.notna() | self.empresas.administracion.notna()].str.upper()
         
-        self._empresas_bop = self.empresas.loc[flt_bop,'Empresa'].to_list()
-        self._empresas_gen = self.empresas.loc[flt_gen | flt_opL | flt_mantL,'Empresa'].to_list()
-        self._empresas_grid = self.empresas.loc[flt_traspo | flt_distro | flt_adm,'Empresa'].to_list()
+        self._empresas_gen = {x:'GENERATOR' for x in self._empresas_gen.to_list()} | {x:'GENERATOR' for x in ['GOLDWIND (SERVICES)', 'VESTAS', 'BLC CROM']}
+        self._empresas_bop = {x:'BOP_CONTRACTOR' for x in self._empresas_bop.to_list()}
+        self._empresas_grid = {x:'GRID_OPERATOR' for x in self._empresas_grid.to_list()}
         
-        self._empresas_gen = {x:'GENERATOR' for x in self._empresas_gen}
-        self._empresas_bop = {x:'BOP_CONTRACTOR' for x in self._empresas_bop}
-        self._empresas_grid = {x:'GRID_OPERATOR' for x in self._empresas_grid}
+        if devolver:
+            return self._empresas_gen | self._empresas_bop | self._empresas_grid | {np.nan: np.nan, pd.NA: pd.NA} 
     
     def __obtener_equipos_parque(self,filtro,valor,potencia=False):
         #El valor del filtro (sea un nemo o una UC), 
@@ -398,268 +393,125 @@ class TablasVC(SQLConnector):
     
     def __procesar_incidencias_it1(self,mensajes=False):
         
-        #Crear el dataframe de incidencias y la estructura que tendrá hasta el final
-        df_tipoequipo = self.tipoequipo\
-                            .loc[:,['idtipoEquipo','id_central','equipo','idTipoGenerador','Pnom']]
-                            
-        df = pd\
-                            .merge(
-                                left=self.incidencias_todas,
-                                right=df_tipoequipo,
-                                left_on=['id_tipoEquipo','id_central'],
-                                right_on=['idtipoEquipo','id_central'],
-                                how='left')\
-                            .drop(columns=['id_tipoEquipo','idtipoEquipo'])
-                            
-        mapeo_owner = self._crear_dict(self.centralesempresas,'idCentral','generador')
-        mapeo_origen = self._crear_dict(self.origen,'idorigen','origen')
-        mapeo_origen2 = {'Interno':'INT','Externo':'EXT','--':'NA'}
-        mapeo_empresas = self._crear_dict(self.empresas,'id','Empresa')
-        mapeo_razones = self._crear_dict(self.razones,'idrazones','razones')
-        mapeo_ucs = self._crear_dict(self.central,'idcentral','unidadComercial')
-        mapeo_nemos = self._crear_dict(self.central,'idcentral','nemoCammesa')
-        mapeo_estado = self._crear_dict(self.estado,'idestado','estado')
-        mapeo_tipogenerador = self._crear_dict(self.tipogenerador,'id','nombre')
-        
-        
-        mapeo_razones2= {
-            'MAPRO':'MAPRO',
-            'MAPRO (no computa)':'MAPRO S_A',
-            'Mantenimiento no programado':'MANOPRO',
-            'Falla':'FAILURE',
-            'Limitación':'LIMITATION',
-            'Reactivo Nocturno':'QNIGHT',
-            'Suspensdido':'SUSPENDED',
-            'Fuerza mayor':'FORCE MAJEURE'
-            }
-        
-        df['UC'] = df['id_central'].map(mapeo_ucs)
-        df['Nemo'] = df['id_central'].map(mapeo_nemos)
-        df['idTipoGenerador'] = df['idTipoGenerador'].map(mapeo_tipogenerador)
-        df['id_trabajo'] = df['id_trabajo'].map(mapeo_empresas)
-        df['id_central'] = df['id_central'].map(mapeo_owner)
-        df['id_razones'] = df['id_razones'].map(mapeo_razones).map(mapeo_razones2)
-        df['id_origen'] = df['id_origen'].map(mapeo_origen).map(mapeo_origen2)
-        df['id_estado'] = df['id_estado'].map(mapeo_estado)
-        df['SolverAgent'] = np.nan
-        df['numero_pt11'] = df['numero_pt11'].replace(to_replace=-1,value=0)
-    
-        #Renombrar columnas
-        cols_renombrar = {
-            'idincidencia':'ID',
-            'idPrimario':'ID_prim',
-            'idLimitacion':'ID_lim',
-            
-            'id_estado':'Status',
-            'UC':'UC',
-            'Nemo':'Nemo',
-            'id_central':'Owner',
-            'cantEquipos':'QtyTot',
-            
-            'evStFecha':'Start',
-            'evStUser':'StartUsr',
-            'evStPotCor':'PowerCut',
+        #Crear el dataframe de incidencias y la estructura que tendrá hasta el final                
+        self.incidencias_todas = (pd
+            .merge(
+                left=self.incidencias_todas,
+                right=self.tipoequipo.loc[:,['idtipoEquipo','id_central','equipo','idTipoGenerador','Pnom']],
+                left_on=['id_tipoEquipo','id_central'],
+                right_on=['idtipoEquipo','id_central'],
+                how='left')
+            .drop(columns=['id_tipoEquipo','idtipoEquipo'])
+            .assign(**{
+                'UC': lambda df_: df_['id_central'].map(self._crear_dict(self.central,'idcentral','unidadComercial')),
+                'Nemo': lambda df_: df_['UC'].map(self._crear_dict(self.central,'unidadComercial','nemoCammesa'),na_action='ignore').fillna('DESCONOIDO'),
+                'idTipoGenerador': lambda df_: df_['idTipoGenerador'].map(self._crear_dict(self.tipogenerador,'id','nombre')),
+                'id_trabajo': lambda df_: df_['id_trabajo'].map(self._crear_dict(self.empresas,'id','Empresa')),
+                'id_central': lambda df_: df_['id_central'].map(self._crear_dict(self.centralesempresas,'idCentral','generador')),
+                'id_estado': lambda df_: df_['id_estado'].map(self._crear_dict(self.estado,'idestado','estado')),
+                'numero_pt11': lambda df_: df_['numero_pt11'].replace(to_replace=-1,value=0),
+                'id_origen': lambda df_: df_['id_origen']
+                                            .map(self._crear_dict(self.origen,'idorigen','origen'))
+                                            .map({'Interno':'INT','Externo':'EXT','--':'NA'}),
+                'id_razones': lambda df_: df_['id_razones']
+                                            .map(self._crear_dict(self.razones,'idrazones','razones'))
+                                            .map({
+                                                'MAPRO':'MAPRO',
+                                                'MAPRO (no computa)':'MAPRO S_A',
+                                                'Mantenimiento no programado':'MANOPRO',
+                                                'Falla':'FAILURE',
+                                                'Limitación':'LIMITATION',
+                                                'Reactivo Nocturno':'QNIGHT',
+                                                'Suspensdido':'SUSPENDED',
+                                                'Fuerza mayor':'FORCE MAJEURE'}),})
+            .rename(columns={
+                    'idincidencia':'ID','idPrimario':'ID_prim','idLimitacion':'ID_lim',
+                    'id_estado':'Status','UC':'UC','Nemo':'Nemo','id_central':'Owner','cantEquipos':'QtyTot',
+                    'evStFecha':'Start','evStUser':'StartUsr','evStPotCor':'PowerCut',
+                    'cuNoFecha':'NoticeDate','evEndFecha':'End','evEndUser':'EndUsr','evEndPot':'PowerRet',
+                    'id_pt11':'PT11','numero_pt11':'ID_PT11',
+                    'equipo':'Equipo','Pnom':'Pnom','idTipoGenerador':'GenType','afCantEquipos':'Qty','afQuantity':'QtyProp',
+                    'afTime':'Hours','energyLoss':'ENS',
+                    'id_trabajo':'SolvedBy','id_razones':'Reason','id_origen':'Origin','codFalla':'Code',
+                    'pot_posible':'Pteo','setpoint_pot':'SP_P',
+                    'descripcion':'BLC_Description','comentario':'BLC_Comments',
+                    'descripcionFalla':'Owner_Description','equipoAfectado':'Owner_AffectedEquipment',
+                    'accionesTomadas':'Owner_ActionsTaken','resultados':'Owner_Results',
+                    'proximosPasos':'Owner_NextSteps','comentariosCliente':'Owner_Comments',}))
 
-            'cuNoFecha':'NoticeDate',
-            
-            'evEndFecha':'End',
-            'evEndUser':'EndUsr',
-            'evEndPot':'PowerRet',
-            
-            'id_pt11':'PT11',
-            'numero_pt11':'ID_PT11',
-            
-            'equipo':'Equipo',
-            'Pnom':'Pnom',
-            'idTipoGenerador':'GenType',
-            'afCantEquipos':'Qty',
-            'afQuantity':'QtyProp',
-            
-            'afTime':'Hours',
-            'energyLoss':'ENS',
-            'id_trabajo':'SolvedBy',
-            'SolverAgent':'SolverAgent',
-            'id_razones':'Reason',
-            'id_origen':'Origin',
-            'codFalla':'Code',
-
-            'pot_posible':'Pteo',
-            'setpoint_pot':'SP_P',
-
-            'descripcion':'BLC_Description',
-            'comentario':'BLC_Comments',
-            
-            'descripcionFalla':'Owner_Description',
-            'equipoAfectado':'Owner_AffectedEquipment',
-            'accionesTomadas':'Owner_ActionsTaken',
-            'resultados':'Owner_Results',
-            'proximosPasos':'Owner_NextSteps',
-            'comentariosCliente':'Owner_Comments',
-        }
-
-
-        self.incidencias_todas = df\
-                                    .rename(columns=cols_renombrar)\
-                                    .copy(deep=True)
- 
     def __procesar_incidencias_it2(self,mensajes=False):
         #Integridad de Datos
-
-        df = self.incidencias_todas
+        #Rellenar huecos, que se dan naturalmente, mapeo de nombres/variables
+        tiene_fecha_descarte = self.incidencias_todas['descartadoFecha'].notna()
+        falta_SolvedBy = self.incidencias_todas['SolvedBy'].isna()
+        falta_EndUsr = self.incidencias_todas.EndUsr.isna()
+        falta_End = self.incidencias_todas.End.isna()
+        falta_Origin = self.incidencias_todas.Origin.isna() | (self.incidencias_todas.Origin.ne('INT') & self.incidencias_todas.Origin.ne('EXT') )
+        falta_Reason = self.incidencias_todas.Reason.eq('NA') | self.incidencias_todas.Reason.isna()
+        no_descartada = self.incidencias_todas.Status.ne('DESCARTADA')
         
-        #Rellenar huecos, que se dan naturalmente
-        descartadas = ~df['descartadoFecha'].isna()
-        falta_SolvedBy = df['SolvedBy'].isna()
-        falta_EndUsr = df['EndUsr'].isna()
-        
-        falta_Origin = (df['Origin'] == 'NA') | (df['Origin'].isna()) | ((df['Origin'] != 'INT') & (df['Origin'] != 'EXT'))
-        falta_Reason = (df['Reason'] == 'NA') | (df['Reason'].isna())
-        flt_no_descartada = df['Status'] != 'DESCARTADA'
-        
-        #Para aquellas incidencias descartadas, se coloca como usuario que cerró, el usuario que descartó.
-        df.loc[descartadas & falta_EndUsr,'EndUsr'] = df.loc[descartadas & falta_EndUsr,'lastModifUser']
-        
-        #Las no descartadas deberían ser incidencias abiertas, por cómo funciona el sistema
-        df.loc[~descartadas & falta_EndUsr,'EndUsr'] = 'corte_automático'
-        df.loc[~descartadas & falta_EndUsr,'End'] = dt.datetime.now().replace(second=0,microsecond=0)
-        
-        #Hay un único caso en el que falta el campo SolvedBy, de una incidencia muy vieja.
-        #Se deja esto aquí para no hardcodear soluciones.
-        df.loc[falta_SolvedBy,'SolvedBy'] = df.loc[falta_SolvedBy,'Owner']
-        
-        #Rellenar huecos de 'Origin' y 'Reason'
-        df.loc[flt_no_descartada & falta_Origin,'Origin'] = 'INT'   # Los huecos y valores extraños se asumen internos
-        df.loc[flt_no_descartada & falta_Reason,'Reason'] = np.nan  # EL hueco se llena más adelante
-
-        #Recalcular el SolverAgent, ya que se llenaron los huecos
-        self.__crear_dict_tipo_empresa()
-        mapeo_solveragent = self._empresas_bop | self._empresas_gen | self._empresas_grid
-        
-        df['SolverAgent'] = df['SolvedBy'].map(mapeo_solveragent)
-        falta_SolverAgent = df['SolverAgent'].isna()
-        
-        df.loc[falta_SolverAgent,'SolverAgent'] = 'INVALIDO'
-        
-        for col in ['Equipo','GenType','Reason']:
-            df[col] = df[col].fillna('DESCONOCIDO')
-            
-        self.incidencias_todas = df.copy(deep=True)
+        self.incidencias_todas = (self.incidencias_todas
+            .assign(
+                EndUsr = lambda df_: df_.EndUsr.mask(tiene_fecha_descarte & falta_EndUsr, df_.lastModifUser),
+                End = lambda df_: df_.End.mask(tiene_fecha_descarte & falta_End, df_.lastModifFecha),)
+            .assign(
+                EndUsr = lambda df_: df_.EndUsr.mask(~tiene_fecha_descarte & falta_EndUsr, 'CORTE_AUTOMATICO'),
+                End = lambda df_: df_.End.mask(~tiene_fecha_descarte & falta_End, fechas.ayer().replace(hour=23,minute=59,second=50)),
+                SolvedBy = lambda df_: df_.SolvedBy.mask(falta_SolvedBy, df_.Owner),
+                Origin = lambda df_: df_.Origin.mask(no_descartada & falta_Origin, 'INT'),
+                Reason = lambda df_: df_.Reason.mask(no_descartada & falta_Reason, np.nan),)
+            .assign(
+                SolverAgent = lambda df_: df_.SolvedBy.str.upper().map(self._crear_dict_tipo_empresa(devolver=True)),
+                Hours = lambda df_: df_.End.sub(df_.Start).dt.total_seconds().div(3600),
+                ENS = lambda df_: df_.ENS.fillna(0))
+            .assign(**{col: lambda df_, c=col: df_[c].fillna('DESCONOCIDO') for col in ['Equipo','GenType','Reason','SolverAgent']}))
 
     def __procesar_incidencias_it3(self,mensajes=False):
         
-        df = self.incidencias_todas
-        
         #Cambiar tipos de datos para ahorrar memoria
-        cols_ui16 = ['PT11','ID_PT11','QtyTot','Qty']
-        cols_ui32 = ['ID','ID_prim','ID_lim',]
-        cols_cat = [
-            'Status',
-            'UC',
-            'Nemo',
-            'Owner',
-            'StartUsr',
-            'EndUsr',
-            'Equipo',
-            'GenType',
-            'SolvedBy',
-            'SolverAgent',
-            'Reason',
-            'Origin',
-            'Code'
-            ]
-
-        for col in cols_ui16:
-            if mensajes: print(f'Convirtiendo {col} a UI16')
-            df[col] = df[col].astype('UInt16')
-            
-        for col in cols_ui32:
-            if mensajes: print(f'Convirtiendo {col} a UI32')
-            df[col] = df[col].astype('UInt32')
-
-        for col in cols_cat:
-            try:
-                df.loc[:,col] = df.loc[:,col].str.upper()
-                valores = df[col].unique()
-                categoria = pd.CategoricalDtype(valores,ordered=False)
-                df.loc[:,col] = df[col].astype(categoria)
+        def col_a_categoria(serie):
+            try: 
+                serie = serie.str.upper()
+                if mensajes: print(f'\tConvirtiendo {serie.name} a categoría')
+                return serie.astype(pd.CategoricalDtype(serie.unique(),ordered=False))
             except:
-                if mensajes: print(f'Procesando Incidencias CROM: Imposible convertir {col} a categoría...')
-        
-        cols_produccion = [
-            'ID',
-            'ID_prim',
-            'ID_lim',
-            
-            'Status',
-            'UC',
-            'Nemo',
-            'Owner',
-            'QtyTot',
-            
-            'Start',
-            'StartUsr',
-            'PowerCut',
+                print(f'\t\tImposible convertir {serie.name} a categoría...')
 
-            'NoticeDate',
-            
-            'End',
-            'EndUsr',
-            'PowerRet',
-            
-            'PT11',
-            'ID_PT11',
-            
-            'Equipo',
-            'Pnom',
-            'GenType',
-            'Qty',
-            'QtyProp',
-            
-            'Hours',
-            'ENS',
-            'SolvedBy',
-            'SolverAgent',
-            'Reason',
-            'Origin',
-            'Code',
+        self.incidencias_todas = (self.incidencias_todas
+            .assign(**{col: lambda df_, c=col,typ=t: df_[c].astype(typ)
+                       for t, cols in {'UInt16':['PT11','ID_PT11','QtyTot','Qty'],
+                                       'UInt32':['ID','ID_prim','ID_lim',]}.items() for col in cols})
 
-            'Pteo',
-            'SP_P',
-
-            'BLC_Description',
-            'BLC_Comments',
-            
-            'Owner_Description',
-            'Owner_AffectedEquipment',
-            'Owner_ActionsTaken',
-            'Owner_Results',
-            'Owner_NextSteps',
-            'Owner_Comments',
-        ]
-
-        cols_auditoria = [ 
-            'sinCuNo',
-            'abiertoFecha',
-            'cerrIncFecha',
-            'cerrComFecha',
-            'visadoFecha',
-            'descartadoFecha',
-            'justificacionDescarte',
-
-            'lastModifUser',
-            'lastModifFecha',
-            'lastModifUserWeb',
-            'lastModifFechaWeb',
-            'comentarioEdicion',
-
-            'resaltar',
-        ]
-        
-        self.incidencias_todas_auditoria = df.loc[:,cols_produccion+cols_auditoria].copy(deep=True)
-        flt_no_descartada = df['Status'] != 'DESCARTADA'
-        self.incidencias_todas = df.loc[flt_no_descartada,cols_produccion].copy(deep=True)
+            .assign(**{col: lambda df_, c=col: col_a_categoria(df_[c]) for col in ['Status','UC','Nemo','Owner',
+                                                                                   'StartUsr','EndUsr','Equipo','GenType',
+                                                                                   'SolvedBy','SolverAgent','Reason','Origin','Code']}))
 
     def __procesar_incidencias_it4(self,mensajes=False):
+        #Separar en incidencias incidencias_todas (no tiene descartadas) de incidencias_todas_auditoria (tiene descartadas)
+        cols_produccion = [
+            'ID','ID_prim','ID_lim',
+            'Status','UC','Nemo','Owner','QtyTot',
+            'Start','StartUsr','PowerCut','NoticeDate','End','EndUsr','PowerRet',
+            'PT11','ID_PT11',
+            'Equipo','Pnom','GenType','Qty','QtyProp',
+            'Hours','ENS',
+            'SolvedBy','SolverAgent','Reason','Origin','Code',
+            'Pteo','SP_P',
+            'BLC_Description','BLC_Comments',
+            'Owner_Description','Owner_AffectedEquipment','Owner_ActionsTaken',
+            'Owner_Results','Owner_NextSteps','Owner_Comments',]
+
+        cols_auditoria = [ 
+            'sinCuNo','abiertoFecha','cerrIncFecha',
+            'cerrComFecha','visadoFecha',
+            'descartadoFecha','justificacionDescarte',
+            'lastModifUser','lastModifFecha','lastModifUserWeb',
+            'lastModifFechaWeb','comentarioEdicion','resaltar',]
+        
+        self.incidencias_todas_auditoria = self.incidencias_todas.loc[:, cols_produccion + cols_auditoria].copy(deep=True)
+        self.incidencias_todas = self.incidencias_todas.loc[self.incidencias_todas.Status.ne('DESCARTADA'), cols_produccion].copy(deep=True)
+    
+    def __procesar_incidencias_it5(self,mensajes=False):
         #Incorporar columnas de la norma IEC61400
         cols_todas = self.incidencias_todas.columns.to_list()
         
@@ -695,20 +547,20 @@ class TablasVC(SQLConnector):
         df_short_merged = pd.concat([df_short_new,df_short_old],ignore_index=True)
         df_long_merged = pd.concat([df_long_new,df_long_old],ignore_index=True)
         
-        if mensajes: print(f"Eliminando duplicados...")
+        if mensajes: print(f"\tEliminando duplicados...")
         subset= ['ID',]
         df_short_merged.drop_duplicates(subset=subset,ignore_index=True,inplace=True)
         df_long_merged.drop_duplicates(subset=subset,ignore_index=True,inplace=True)
         
-        if mensajes: print(f"Reordenando...")
+        if mensajes: print(f"\tReordenando...")
         df_short_merged.sort_values(by=subset,ascending=[True for _ in subset],inplace=True)
         df_long_merged.sort_values(by=subset,ascending=[True for _ in subset],inplace=True)
         
-        if mensajes: print(f"Asignando resultados.")
+        if mensajes: print(f"\tAsignando resultados.")
         self.incidencias_todas = df_short_merged.copy(deep=True)
         self.incidencias_todas_auditoria = df_long_merged.copy(deep=True)
         
-        if mensajes: print(f"Guardando copia local del resultado.")
+        if mensajes: print(f"\tGuardando copia local del resultado.")
         self.__crear_backup_de_las_tablas(datos_basicos=False,incidencias=True,mensajes=mensajes)
 
         del df_short_new,df_short_old,df_long_new,df_long_old,df_short_merged,df_long_merged,dict_dfs
@@ -725,7 +577,8 @@ class TablasVC(SQLConnector):
         self.__procesar_incidencias_it1(mensajes=mensajes)  #Renombrar columnas
         self.__procesar_incidencias_it2(mensajes=mensajes)  #Validación de datos
         self.__procesar_incidencias_it3(mensajes=mensajes)  #Optimización de consumo de memoria
-        self.__procesar_incidencias_it4(mensajes=mensajes)  #Procesar incidencias bajo IEC61400
+        self.__procesar_incidencias_it4(mensajes=mensajes)  #Separar en incidencias_todas e incidencias_todas_auditoria
+        self.__procesar_incidencias_it5(mensajes=mensajes)  #Procesar incidencias bajo IEC61400
         
     def __asignar_datos_central(self,mensajes=False):
         self.central = self.dfs['central']
@@ -908,6 +761,8 @@ class TablasVC(SQLConnector):
             self.consultar_ucs(solo_CROM=self.solo_CROM)
             self.consultar_nemoCammesa(solo_CROM=self.solo_CROM)
         
+        self._crear_dict_tipo_empresa()
+        
         if desconectar:
             self.desconectar(mensajes=mensajes)
 
@@ -932,7 +787,7 @@ class TablasVC(SQLConnector):
             fecha_i,fecha_f = fechas.validar_fechas(fecha_ini = fecha_i, fecha_fin = fecha_f)
             
         if query is None:
-            query = f"SELECT * FROM incidencias WHERE evStFecha <= '{fecha_f}' AND evEndFecha >= '{fecha_i}'"
+            query = f"SELECT * FROM incidencias WHERE evStFecha <= '{fecha_f}' AND (evEndFecha >= '{fecha_i}' OR evEndFecha IS NULL)"
         
         if mensajes: (f"Consultando incidencias:")
         if mensajes: (f"Query: {query}")
@@ -1003,6 +858,7 @@ class TablasVC(SQLConnector):
                 self.consultar_ucs(solo_CROM=self.solo_CROM)
                 self.consultar_nemoCammesa(solo_CROM=self.solo_CROM)
                 
+                self._crear_dict_tipo_empresa()
             else:
                 print(f"No se encontró un archivo con los datos básicos offline {self.__archivo_tablas}")
         
@@ -1110,4 +966,20 @@ class TablasVC(SQLConnector):
 
         if inc_on: 
             self.__consultar_incidencias(mensajes=mensajes,fecha_i=fecha_i,fecha_f=fecha_f)
-            self.__crear_backup_de_las_tablas(datos_basicos=False,incidencias=True)
\ No newline at end of file
+            self.__crear_backup_de_las_tablas(datos_basicos=False,incidencias=True)
+            
+    def modificar_incidencia(self,id,**kwargs):
+        
+        if self.incidencias_todas is None:
+            raise Exception(f'No se han cargado incidencias aún.')
+        
+        vars_invalidas = [x for x in kwargs.keys() if x not in self.incidencias_todas.columns]
+        if vars_invalidas:
+            raise KeyError(f'Las variables {vars_invalidas} no se encontraron en las columnas del DataFrame de incidencias {self.incidencias_todas.columns.to_list()}')
+        
+        if not self.incidencias_todas.query(f'ID == {id}').empty:
+            for k,v in kwargs.items():
+                self.incidencias_todas.loc[self.incidencias_todas.ID.eq(id), k] = v
+                self.incidencias_todas_auditoria.loc[self.incidencias_todas_auditoria.ID.eq(id), k] = v
+        else:
+            raise ValueError(f'No se encontró la incidencia con ID {id}')
\ No newline at end of file
